@@ -5,30 +5,31 @@ estimator.
 # Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
 # License: BSD 3 clause
 # Copyright: INRIA
-import warnings
 import operator
 import sys
 import time
-
+import warnings
 from numbers import Integral, Real
+
 import numpy as np
 from scipy import linalg
 
-from . import empirical_covariance, EmpiricalCovariance, log_likelihood
-
+from ..base import _fit_context
 from ..exceptions import ConvergenceWarning
-from ..utils.validation import (
-    _is_arraylike_not_scalar,
-    check_random_state,
-    check_scalar,
-)
-from ..utils.parallel import delayed, Parallel
-from ..utils._param_validation import Interval, StrOptions
 
 # mypy error: Module 'sklearn.linear_model' has no attribute '_cd_fast'
 from ..linear_model import _cd_fast as cd_fast  # type: ignore
 from ..linear_model import lars_path_gram
 from ..model_selection import check_cv, cross_val_score
+from ..utils._param_validation import Interval, StrOptions, validate_params
+from ..utils.metadata_routing import _RoutingNotSupportedMixin
+from ..utils.parallel import Parallel, delayed
+from ..utils.validation import (
+    _is_arraylike_not_scalar,
+    check_random_state,
+    check_scalar,
+)
+from . import EmpiricalCovariance, empirical_covariance, log_likelihood
 
 
 # Helper functions to compute the objective and dual objective functions
@@ -58,27 +59,8 @@ def _dual_gap(emp_cov, precision_, alpha):
     return gap
 
 
-def alpha_max(emp_cov):
-    """Find the maximum alpha for which there are some non-zeros off-diagonal.
-
-    Parameters
-    ----------
-    emp_cov : ndarray of shape (n_features, n_features)
-        The sample covariance matrix.
-
-    Notes
-    -----
-    This results from the bound for the all the Lasso that are solved
-    in GraphicalLasso: each time, the row of cov corresponds to Xy. As the
-    bound for alpha is given by `max(abs(Xy))`, the result follows.
-    """
-    A = np.copy(emp_cov)
-    A.flat[:: A.shape[0] + 1] = 0
-    return np.max(np.abs(A))
-
-
 # The g-lasso algorithm
-def graphical_lasso(
+def _graphical_lasso(
     emp_cov,
     alpha,
     *,
@@ -88,112 +70,17 @@ def graphical_lasso(
     enet_tol=1e-4,
     max_iter=100,
     verbose=False,
-    return_costs=False,
     eps=np.finfo(np.float64).eps,
-    return_n_iter=False,
 ):
-    """L1-penalized covariance estimator.
-
-    Read more in the :ref:`User Guide <sparse_inverse_covariance>`.
-
-    .. versionchanged:: v0.20
-        graph_lasso has been renamed to graphical_lasso
-
-    Parameters
-    ----------
-    emp_cov : ndarray of shape (n_features, n_features)
-        Empirical covariance from which to compute the covariance estimate.
-
-    alpha : float
-        The regularization parameter: the higher alpha, the more
-        regularization, the sparser the inverse covariance.
-        Range is (0, inf].
-
-    cov_init : array of shape (n_features, n_features), default=None
-        The initial guess for the covariance. If None, then the empirical
-        covariance is used.
-
-    mode : {'cd', 'lars'}, default='cd'
-        The Lasso solver to use: coordinate descent or LARS. Use LARS for
-        very sparse underlying graphs, where p > n. Elsewhere prefer cd
-        which is more numerically stable.
-
-    tol : float, default=1e-4
-        The tolerance to declare convergence: if the dual gap goes below
-        this value, iterations are stopped. Range is (0, inf].
-
-    enet_tol : float, default=1e-4
-        The tolerance for the elastic net solver used to calculate the descent
-        direction. This parameter controls the accuracy of the search direction
-        for a given column update, not of the overall parameter estimate. Only
-        used for mode='cd'. Range is (0, inf].
-
-    max_iter : int, default=100
-        The maximum number of iterations.
-
-    verbose : bool, default=False
-        If verbose is True, the objective function and dual gap are
-        printed at each iteration.
-
-    return_costs : bool, default=False
-        If return_costs is True, the objective function and dual gap
-        at each iteration are returned.
-
-    eps : float, default=eps
-        The machine-precision regularization in the computation of the
-        Cholesky diagonal factors. Increase this for very ill-conditioned
-        systems. Default is `np.finfo(np.float64).eps`.
-
-    return_n_iter : bool, default=False
-        Whether or not to return the number of iterations.
-
-    Returns
-    -------
-    covariance : ndarray of shape (n_features, n_features)
-        The estimated covariance matrix.
-
-    precision : ndarray of shape (n_features, n_features)
-        The estimated (sparse) precision matrix.
-
-    costs : list of (objective, dual_gap) pairs
-        The list of values of the objective function and the dual gap at
-        each iteration. Returned only if return_costs is True.
-
-    n_iter : int
-        Number of iterations. Returned only if `return_n_iter` is set to True.
-
-    See Also
-    --------
-    GraphicalLasso : Sparse inverse covariance estimation
-        with an l1-penalized estimator.
-    GraphicalLassoCV : Sparse inverse covariance with
-        cross-validated choice of the l1 penalty.
-
-    Notes
-    -----
-    The algorithm employed to solve this problem is the GLasso algorithm,
-    from the Friedman 2008 Biostatistics paper. It is the same algorithm
-    as in the R `glasso` package.
-
-    One possible difference with the `glasso` R package is that the
-    diagonal coefficients are not penalized.
-    """
     _, n_features = emp_cov.shape
     if alpha == 0:
-        if return_costs:
-            precision_ = linalg.inv(emp_cov)
-            cost = -2.0 * log_likelihood(emp_cov, precision_)
-            cost += n_features * np.log(2 * np.pi)
-            d_gap = np.sum(emp_cov * precision_) - n_features
-            if return_n_iter:
-                return emp_cov, precision_, (cost, d_gap), 0
-            else:
-                return emp_cov, precision_, (cost, d_gap)
-        else:
-            if return_n_iter:
-                return emp_cov, linalg.inv(emp_cov), 0
-            else:
-                return emp_cov, linalg.inv(emp_cov)
+        # Early return without regularization
+        precision_ = linalg.inv(emp_cov)
+        cost = -2.0 * log_likelihood(emp_cov, precision_)
+        cost += n_features * np.log(2 * np.pi)
+        d_gap = np.sum(emp_cov * precision_) - n_features
+        return emp_cov, precision_, (cost, d_gap), 0
+
     if cov_init is None:
         covariance_ = emp_cov.copy()
     else:
@@ -210,6 +97,7 @@ def graphical_lasso(
     precision_ = linalg.pinvh(covariance_)
 
     indices = np.arange(n_features)
+    i = 0  # initialize the counter to be robust to `max_iter=0`
     costs = list()
     # The different l1 regression solver have different numerical errors
     if mode == "cd":
@@ -285,8 +173,7 @@ def graphical_lasso(
                     "[graphical_lasso] Iteration % 3i, cost % 3.2e, dual gap %.3e"
                     % (i, cost, d_gap)
                 )
-            if return_costs:
-                costs.append((cost, d_gap))
+            costs.append((cost, d_gap))
             if np.abs(d_gap) < tol:
                 break
             if not np.isfinite(cost) and i > 0:
@@ -303,16 +190,169 @@ def graphical_lasso(
         e.args = (e.args[0] + ". The system is too ill-conditioned for this solver",)
         raise e
 
+    return covariance_, precision_, costs, i + 1
+
+
+def alpha_max(emp_cov):
+    """Find the maximum alpha for which there are some non-zeros off-diagonal.
+
+    Parameters
+    ----------
+    emp_cov : ndarray of shape (n_features, n_features)
+        The sample covariance matrix.
+
+    Notes
+    -----
+    This results from the bound for the all the Lasso that are solved
+    in GraphicalLasso: each time, the row of cov corresponds to Xy. As the
+    bound for alpha is given by `max(abs(Xy))`, the result follows.
+    """
+    A = np.copy(emp_cov)
+    A.flat[:: A.shape[0] + 1] = 0
+    return np.max(np.abs(A))
+
+
+@validate_params(
+    {
+        "emp_cov": ["array-like"],
+        "cov_init": ["array-like", None],
+        "return_costs": ["boolean"],
+        "return_n_iter": ["boolean"],
+    },
+    prefer_skip_nested_validation=False,
+)
+def graphical_lasso(
+    emp_cov,
+    alpha,
+    *,
+    cov_init=None,
+    mode="cd",
+    tol=1e-4,
+    enet_tol=1e-4,
+    max_iter=100,
+    verbose=False,
+    return_costs=False,
+    eps=np.finfo(np.float64).eps,
+    return_n_iter=False,
+):
+    """L1-penalized covariance estimator.
+
+    Read more in the :ref:`User Guide <sparse_inverse_covariance>`.
+
+    .. versionchanged:: v0.20
+        graph_lasso has been renamed to graphical_lasso
+
+    Parameters
+    ----------
+    emp_cov : array-like of shape (n_features, n_features)
+        Empirical covariance from which to compute the covariance estimate.
+
+    alpha : float
+        The regularization parameter: the higher alpha, the more
+        regularization, the sparser the inverse covariance.
+        Range is (0, inf].
+
+    cov_init : array of shape (n_features, n_features), default=None
+        The initial guess for the covariance. If None, then the empirical
+        covariance is used.
+
+        .. deprecated:: 1.3
+           `cov_init` is deprecated in 1.3 and will be removed in 1.5.
+           It currently has no effect.
+
+    mode : {'cd', 'lars'}, default='cd'
+        The Lasso solver to use: coordinate descent or LARS. Use LARS for
+        very sparse underlying graphs, where p > n. Elsewhere prefer cd
+        which is more numerically stable.
+
+    tol : float, default=1e-4
+        The tolerance to declare convergence: if the dual gap goes below
+        this value, iterations are stopped. Range is (0, inf].
+
+    enet_tol : float, default=1e-4
+        The tolerance for the elastic net solver used to calculate the descent
+        direction. This parameter controls the accuracy of the search direction
+        for a given column update, not of the overall parameter estimate. Only
+        used for mode='cd'. Range is (0, inf].
+
+    max_iter : int, default=100
+        The maximum number of iterations.
+
+    verbose : bool, default=False
+        If verbose is True, the objective function and dual gap are
+        printed at each iteration.
+
+    return_costs : bool, default=False
+        If return_costs is True, the objective function and dual gap
+        at each iteration are returned.
+
+    eps : float, default=eps
+        The machine-precision regularization in the computation of the
+        Cholesky diagonal factors. Increase this for very ill-conditioned
+        systems. Default is `np.finfo(np.float64).eps`.
+
+    return_n_iter : bool, default=False
+        Whether or not to return the number of iterations.
+
+    Returns
+    -------
+    covariance : ndarray of shape (n_features, n_features)
+        The estimated covariance matrix.
+
+    precision : ndarray of shape (n_features, n_features)
+        The estimated (sparse) precision matrix.
+
+    costs : list of (objective, dual_gap) pairs
+        The list of values of the objective function and the dual gap at
+        each iteration. Returned only if return_costs is True.
+
+    n_iter : int
+        Number of iterations. Returned only if `return_n_iter` is set to True.
+
+    See Also
+    --------
+    GraphicalLasso : Sparse inverse covariance estimation
+        with an l1-penalized estimator.
+    GraphicalLassoCV : Sparse inverse covariance with
+        cross-validated choice of the l1 penalty.
+
+    Notes
+    -----
+    The algorithm employed to solve this problem is the GLasso algorithm,
+    from the Friedman 2008 Biostatistics paper. It is the same algorithm
+    as in the R `glasso` package.
+
+    One possible difference with the `glasso` R package is that the
+    diagonal coefficients are not penalized.
+    """
+
+    if cov_init is not None:
+        warnings.warn(
+            (
+                "The cov_init parameter is deprecated in 1.3 and will be removed in "
+                "1.5. It does not have any effect."
+            ),
+            FutureWarning,
+        )
+
+    model = GraphicalLasso(
+        alpha=alpha,
+        mode=mode,
+        covariance="precomputed",
+        tol=tol,
+        enet_tol=enet_tol,
+        max_iter=max_iter,
+        verbose=verbose,
+        eps=eps,
+        assume_centered=True,
+    ).fit(emp_cov)
+
+    output = [model.covariance_, model.precision_]
     if return_costs:
-        if return_n_iter:
-            return covariance_, precision_, costs, i + 1
-        else:
-            return covariance_, precision_, costs
-    else:
-        if return_n_iter:
-            return covariance_, precision_, i + 1
-        else:
-            return covariance_, precision_
+        output.append(model.costs_)
+    if return_n_iter:
+        output.append(model.n_iter_)
+    return tuple(output)
 
 
 class BaseGraphicalLasso(EmpiricalCovariance):
@@ -323,6 +363,7 @@ class BaseGraphicalLasso(EmpiricalCovariance):
         "max_iter": [Interval(Integral, 0, None, closed="left")],
         "mode": [StrOptions({"cd", "lars"})],
         "verbose": ["verbose"],
+        "eps": [Interval(Real, 0, None, closed="both")],
     }
     _parameter_constraints.pop("store_precision")
 
@@ -333,6 +374,7 @@ class BaseGraphicalLasso(EmpiricalCovariance):
         max_iter=100,
         mode="cd",
         verbose=False,
+        eps=np.finfo(np.float64).eps,
         assume_centered=False,
     ):
         super().__init__(assume_centered=assume_centered)
@@ -341,6 +383,7 @@ class BaseGraphicalLasso(EmpiricalCovariance):
         self.max_iter = max_iter
         self.mode = mode
         self.verbose = verbose
+        self.eps = eps
 
 
 class GraphicalLasso(BaseGraphicalLasso):
@@ -363,6 +406,13 @@ class GraphicalLasso(BaseGraphicalLasso):
         very sparse underlying graphs, where p > n. Elsewhere prefer cd
         which is more numerically stable.
 
+    covariance : "precomputed", default=None
+        If covariance is "precomputed", the input data in `fit` is assumed
+        to be the covariance matrix. If `None`, the empirical covariance
+        is estimated from the data `X`.
+
+        .. versionadded:: 1.3
+
     tol : float, default=1e-4
         The tolerance to declare convergence: if the dual gap goes below
         this value, iterations are stopped. Range is (0, inf].
@@ -380,6 +430,13 @@ class GraphicalLasso(BaseGraphicalLasso):
         If verbose is True, the objective function and dual gap are
         plotted at each iteration.
 
+    eps : float, default=eps
+        The machine-precision regularization in the computation of the
+        Cholesky diagonal factors. Increase this for very ill-conditioned
+        systems. Default is `np.finfo(np.float64).eps`.
+
+        .. versionadded:: 1.3
+
     assume_centered : bool, default=False
         If True, data are not centered before computation.
         Useful when working with data whose mean is almost, but not exactly
@@ -400,6 +457,12 @@ class GraphicalLasso(BaseGraphicalLasso):
     n_iter_ : int
         Number of iterations run.
 
+    costs_ : list of (objective, dual_gap) pairs
+        The list of values of the objective function and the dual gap at
+        each iteration. Returned only if return_costs is True.
+
+        .. versionadded:: 1.3
+
     n_features_in_ : int
         Number of features seen during :term:`fit`.
 
@@ -441,7 +504,8 @@ class GraphicalLasso(BaseGraphicalLasso):
 
     _parameter_constraints: dict = {
         **BaseGraphicalLasso._parameter_constraints,
-        "alpha": [Interval(Real, 0, None, closed="right")],
+        "alpha": [Interval(Real, 0, None, closed="both")],
+        "covariance": [StrOptions({"precomputed"}), None],
     }
 
     def __init__(
@@ -449,10 +513,12 @@ class GraphicalLasso(BaseGraphicalLasso):
         alpha=0.01,
         *,
         mode="cd",
+        covariance=None,
         tol=1e-4,
         enet_tol=1e-4,
         max_iter=100,
         verbose=False,
+        eps=np.finfo(np.float64).eps,
         assume_centered=False,
     ):
         super().__init__(
@@ -461,10 +527,13 @@ class GraphicalLasso(BaseGraphicalLasso):
             max_iter=max_iter,
             mode=mode,
             verbose=verbose,
+            eps=eps,
             assume_centered=assume_centered,
         )
         self.alpha = alpha
+        self.covariance = covariance
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y=None):
         """Fit the GraphicalLasso model to X.
 
@@ -481,24 +550,29 @@ class GraphicalLasso(BaseGraphicalLasso):
         self : object
             Returns the instance itself.
         """
-        self._validate_params()
         # Covariance does not make sense for a single feature
         X = self._validate_data(X, ensure_min_features=2, ensure_min_samples=2)
 
-        if self.assume_centered:
+        if self.covariance == "precomputed":
+            emp_cov = X.copy()
             self.location_ = np.zeros(X.shape[1])
         else:
-            self.location_ = X.mean(0)
-        emp_cov = empirical_covariance(X, assume_centered=self.assume_centered)
-        self.covariance_, self.precision_, self.n_iter_ = graphical_lasso(
+            emp_cov = empirical_covariance(X, assume_centered=self.assume_centered)
+            if self.assume_centered:
+                self.location_ = np.zeros(X.shape[1])
+            else:
+                self.location_ = X.mean(0)
+
+        self.covariance_, self.precision_, self.costs_, self.n_iter_ = _graphical_lasso(
             emp_cov,
             alpha=self.alpha,
+            cov_init=None,
             mode=self.mode,
             tol=self.tol,
             enet_tol=self.enet_tol,
             max_iter=self.max_iter,
             verbose=self.verbose,
-            return_n_iter=True,
+            eps=self.eps,
         )
         return self
 
@@ -514,6 +588,7 @@ def graphical_lasso_path(
     enet_tol=1e-4,
     max_iter=100,
     verbose=False,
+    eps=np.finfo(np.float64).eps,
 ):
     """l1-penalized covariance estimator along a path of decreasing alphas
 
@@ -557,6 +632,13 @@ def graphical_lasso_path(
         The higher the verbosity flag, the more information is printed
         during the fitting.
 
+    eps : float, default=eps
+        The machine-precision regularization in the computation of the
+        Cholesky diagonal factors. Increase this for very ill-conditioned
+        systems. Default is `np.finfo(np.float64).eps`.
+
+        .. versionadded:: 1.3
+
     Returns
     -------
     covariances_ : list of shape (n_alphas,) of ndarray of shape \
@@ -586,7 +668,7 @@ def graphical_lasso_path(
     for alpha in alphas:
         try:
             # Capture the errors, and move on
-            covariance_, precision_ = graphical_lasso(
+            covariance_, precision_, _, _ = _graphical_lasso(
                 emp_cov,
                 alpha=alpha,
                 cov_init=covariance_,
@@ -595,6 +677,7 @@ def graphical_lasso_path(
                 enet_tol=enet_tol,
                 max_iter=max_iter,
                 verbose=inner_verbose,
+                eps=eps,
             )
             covariances_.append(covariance_)
             precisions_.append(precision_)
@@ -623,7 +706,7 @@ def graphical_lasso_path(
     return covariances_, precisions_
 
 
-class GraphicalLassoCV(BaseGraphicalLasso):
+class GraphicalLassoCV(_RoutingNotSupportedMixin, BaseGraphicalLasso):
     """Sparse inverse covariance w/ cross-validated choice of the l1 penalty.
 
     See glossary entry for :term:`cross-validation estimator`.
@@ -655,7 +738,7 @@ class GraphicalLassoCV(BaseGraphicalLasso):
         - :term:`CV splitter`,
         - An iterable yielding (train, test) splits as arrays of indices.
 
-        For integer/None inputs :class:`KFold` is used.
+        For integer/None inputs :class:`~sklearn.model_selection.KFold` is used.
 
         Refer :ref:`User Guide <cross_validation>` for the various
         cross-validation strategies that can be used here.
@@ -695,6 +778,13 @@ class GraphicalLassoCV(BaseGraphicalLasso):
         If verbose is True, the objective function and duality gap are
         printed at each iteration.
 
+    eps : float, default=eps
+        The machine-precision regularization in the computation of the
+        Cholesky diagonal factors. Increase this for very ill-conditioned
+        systems. Default is `np.finfo(np.float64).eps`.
+
+        .. versionadded:: 1.3
+
     assume_centered : bool, default=False
         If True, data are not centered before computation.
         Useful when working with data whose mean is almost, but not exactly
@@ -712,6 +802,12 @@ class GraphicalLassoCV(BaseGraphicalLasso):
     precision_ : ndarray of shape (n_features, n_features)
         Estimated precision matrix (inverse covariance).
 
+    costs_ : list of (objective, dual_gap) pairs
+        The list of values of the objective function and the dual gap at
+        each iteration. Returned only if return_costs is True.
+
+        .. versionadded:: 1.3
+
     alpha_ : float
         Penalization parameter selected.
 
@@ -795,7 +891,7 @@ class GraphicalLassoCV(BaseGraphicalLasso):
 
     _parameter_constraints: dict = {
         **BaseGraphicalLasso._parameter_constraints,
-        "alphas": [Interval(Integral, 1, None, closed="left"), "array-like"],
+        "alphas": [Interval(Integral, 0, None, closed="left"), "array-like"],
         "n_refinements": [Interval(Integral, 1, None, closed="left")],
         "cv": ["cv_object"],
         "n_jobs": [Integral, None],
@@ -813,6 +909,7 @@ class GraphicalLassoCV(BaseGraphicalLasso):
         mode="cd",
         n_jobs=None,
         verbose=False,
+        eps=np.finfo(np.float64).eps,
         assume_centered=False,
     ):
         super().__init__(
@@ -821,6 +918,7 @@ class GraphicalLassoCV(BaseGraphicalLasso):
             max_iter=max_iter,
             mode=mode,
             verbose=verbose,
+            eps=eps,
             assume_centered=assume_centered,
         )
         self.alphas = alphas
@@ -828,6 +926,7 @@ class GraphicalLassoCV(BaseGraphicalLasso):
         self.cv = cv
         self.n_jobs = n_jobs
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y=None):
         """Fit the GraphicalLasso covariance model to X.
 
@@ -844,7 +943,6 @@ class GraphicalLassoCV(BaseGraphicalLasso):
         self : object
             Returns the instance itself.
         """
-        self._validate_params()
         # Covariance does not make sense for a single feature
         X = self._validate_data(X, ensure_min_features=2)
         if self.assume_centered:
@@ -900,6 +998,7 @@ class GraphicalLassoCV(BaseGraphicalLasso):
                         enet_tol=self.enet_tol,
                         max_iter=int(0.1 * self.max_iter),
                         verbose=inner_verbose,
+                        eps=self.eps,
                     )
                     for train, test in cv.split(X, y)
                 )
@@ -983,7 +1082,7 @@ class GraphicalLassoCV(BaseGraphicalLasso):
         self.alpha_ = best_alpha
 
         # Finally fit the model with the selected alpha
-        self.covariance_, self.precision_, self.n_iter_ = graphical_lasso(
+        self.covariance_, self.precision_, self.costs_, self.n_iter_ = _graphical_lasso(
             emp_cov,
             alpha=best_alpha,
             mode=self.mode,
@@ -991,6 +1090,6 @@ class GraphicalLassoCV(BaseGraphicalLasso):
             enet_tol=self.enet_tol,
             max_iter=self.max_iter,
             verbose=inner_verbose,
-            return_n_iter=True,
+            eps=self.eps,
         )
         return self
