@@ -1,32 +1,39 @@
-import pytest
+import io
 import warnings
+from itertools import product
 
 import numpy as np
+import pytest
 from scipy import sparse
 from scipy.stats import kstest
 
-import io
-
-from sklearn.utils._testing import assert_allclose
-from sklearn.utils._testing import assert_allclose_dense_sparse
-from sklearn.utils._testing import assert_array_equal
-from sklearn.utils._testing import assert_array_almost_equal
+from sklearn import tree
+from sklearn.datasets import load_diabetes
+from sklearn.dummy import DummyRegressor
+from sklearn.exceptions import ConvergenceWarning
 
 # make IterativeImputer available
 from sklearn.experimental import enable_iterative_imputer  # noqa
-
-from sklearn.datasets import load_diabetes
-from sklearn.impute import MissingIndicator
-from sklearn.impute import SimpleImputer, IterativeImputer
-from sklearn.dummy import DummyRegressor
-from sklearn.linear_model import BayesianRidge, ARDRegression, RidgeCV
-from sklearn.pipeline import Pipeline
-from sklearn.pipeline import make_union
-from sklearn.model_selection import GridSearchCV
-from sklearn import tree
-from sklearn.random_projection import _sparse_random_matrix
-from sklearn.exceptions import ConvergenceWarning
+from sklearn.impute import IterativeImputer, KNNImputer, MissingIndicator, SimpleImputer
 from sklearn.impute._base import _most_frequent
+from sklearn.linear_model import ARDRegression, BayesianRidge, RidgeCV
+from sklearn.model_selection import GridSearchCV
+from sklearn.pipeline import Pipeline, make_union
+from sklearn.random_projection import _sparse_random_matrix
+from sklearn.utils._testing import (
+    _convert_container,
+    assert_allclose,
+    assert_allclose_dense_sparse,
+    assert_array_almost_equal,
+    assert_array_equal,
+)
+from sklearn.utils.fixes import (
+    BSR_CONTAINERS,
+    COO_CONTAINERS,
+    CSC_CONTAINERS,
+    CSR_CONTAINERS,
+    LIL_CONTAINERS,
+)
 
 
 def _assert_array_equal_and_same_dtype(x, y):
@@ -39,7 +46,9 @@ def _assert_allclose_and_same_dtype(x, y):
     assert x.dtype == y.dtype
 
 
-def _check_statistics(X, X_true, strategy, statistics, missing_values):
+def _check_statistics(
+    X, X_true, strategy, statistics, missing_values, sparse_container
+):
     """Utility function for testing imputation for a given strategy.
 
     Test with dense and sparse arrays
@@ -66,8 +75,8 @@ def _check_statistics(X, X_true, strategy, statistics, missing_values):
 
     # Sparse matrix
     imputer = SimpleImputer(missing_values=missing_values, strategy=strategy)
-    imputer.fit(sparse.csc_matrix(X))
-    X_trans = imputer.transform(sparse.csc_matrix(X.copy()))
+    imputer.fit(sparse_container(X))
+    X_trans = imputer.transform(sparse_container(X.copy()))
 
     if sparse.issparse(X_trans):
         X_trans = X_trans.toarray()
@@ -77,13 +86,14 @@ def _check_statistics(X, X_true, strategy, statistics, missing_values):
 
 
 @pytest.mark.parametrize("strategy", ["mean", "median", "most_frequent", "constant"])
-def test_imputation_shape(strategy):
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_imputation_shape(strategy, csr_container):
     # Verify the shapes of the imputed matrix for different strategies.
     X = np.random.randn(10, 2)
     X[::2] = np.nan
 
     imputer = SimpleImputer(strategy=strategy)
-    X_imputed = imputer.fit_transform(sparse.csr_matrix(X))
+    X_imputed = imputer.fit_transform(csr_container(X))
     assert X_imputed.shape == (10, 2)
     X_imputed = imputer.fit_transform(X)
     assert X_imputed.shape == (10, 2)
@@ -93,25 +103,11 @@ def test_imputation_shape(strategy):
     assert X_imputed.shape == (10, 2)
 
 
-@pytest.mark.parametrize("strategy", ["const", 101, None])
-def test_imputation_error_invalid_strategy(strategy):
-    X = np.ones((3, 5))
-    X[0, 0] = np.nan
-
-    with pytest.raises(ValueError, match=str(strategy)):
-        imputer = SimpleImputer(strategy=strategy)
-        imputer.fit_transform(X)
-
-
 @pytest.mark.parametrize("strategy", ["mean", "median", "most_frequent"])
 def test_imputation_deletion_warning(strategy):
     X = np.ones((3, 5))
     X[:, 0] = np.nan
-    imputer = SimpleImputer(strategy=strategy, verbose=1)
-
-    # TODO: Remove in 1.3
-    with pytest.warns(FutureWarning, match="The 'verbose' parameter"):
-        imputer.fit(X)
+    imputer = SimpleImputer(strategy=strategy).fit(X)
 
     with pytest.warns(UserWarning, match="Skipping"):
         imputer.transform(X)
@@ -119,7 +115,6 @@ def test_imputation_deletion_warning(strategy):
 
 @pytest.mark.parametrize("strategy", ["mean", "median", "most_frequent"])
 def test_imputation_deletion_warning_feature_names(strategy):
-
     pd = pytest.importorskip("pandas")
 
     missing_values = np.nan
@@ -132,11 +127,7 @@ def test_imputation_deletion_warning_feature_names(strategy):
         columns=feature_names,
     )
 
-    imputer = SimpleImputer(strategy=strategy, verbose=1)
-
-    # TODO: Remove in 1.3
-    with pytest.warns(FutureWarning, match="The 'verbose' parameter"):
-        imputer.fit(X)
+    imputer = SimpleImputer(strategy=strategy).fit(X)
 
     # check SimpleImputer returning feature name attribute correctly
     assert_array_equal(imputer.feature_names_in_, feature_names)
@@ -149,11 +140,12 @@ def test_imputation_deletion_warning_feature_names(strategy):
 
 
 @pytest.mark.parametrize("strategy", ["mean", "median", "most_frequent", "constant"])
-def test_imputation_error_sparse_0(strategy):
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_imputation_error_sparse_0(strategy, csc_container):
     # check that error are raised when missing_values = 0 and input is sparse
     X = np.ones((3, 5))
     X[0] = 0
-    X = sparse.csc_matrix(X)
+    X = csc_container(X)
 
     imputer = SimpleImputer(strategy=strategy, missing_values=0)
     with pytest.raises(ValueError, match="Provide a dense array"):
@@ -176,7 +168,8 @@ def safe_mean(arr, *args, **kwargs):
     return np.nan if length == 0 else np.mean(arr, *args, **kwargs)
 
 
-def test_imputation_mean_median():
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_imputation_mean_median(csc_container):
     # Test imputation using the mean and median strategies, when
     # missing_values != 0.
     rng = np.random.RandomState(0)
@@ -240,10 +233,13 @@ def test_imputation_mean_median():
 
         X_true = X_true[:, cols_to_keep]
 
-        _check_statistics(X, X_true, strategy, true_statistics, test_missing_values)
+        _check_statistics(
+            X, X_true, strategy, true_statistics, test_missing_values, csc_container
+        )
 
 
-def test_imputation_median_special_cases():
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_imputation_median_special_cases(csc_container):
     # Test median imputation with sparse boundary cases
     X = np.array(
         [
@@ -272,14 +268,16 @@ def test_imputation_median_special_cases():
     ).transpose()
     statistics_median = [0, 5, 0, -2.5, 2.5, 4.5, -4.5, 0.5]
 
-    _check_statistics(X, X_imputed_median, "median", statistics_median, np.nan)
+    _check_statistics(
+        X, X_imputed_median, "median", statistics_median, np.nan, csc_container
+    )
 
 
 @pytest.mark.parametrize("strategy", ["mean", "median"])
 @pytest.mark.parametrize("dtype", [None, object, str])
 def test_imputation_mean_median_error_invalid_type(strategy, dtype):
     X = np.array([["a", "b", 3], [4, "e", 6], ["g", "h", 9]], dtype=dtype)
-    msg = "non-numeric data:\ncould not convert string to float: '"
+    msg = "non-numeric data:\ncould not convert string to float:"
     with pytest.raises(ValueError, match=msg):
         imputer = SimpleImputer(strategy=strategy)
         imputer.fit_transform(X)
@@ -292,7 +290,7 @@ def test_imputation_mean_median_error_invalid_type_list_pandas(strategy, type):
     if type == "dataframe":
         pd = pytest.importorskip("pandas")
         X = pd.DataFrame(X)
-    msg = "non-numeric data:\ncould not convert string to float: '"
+    msg = "non-numeric data:\ncould not convert string to float:"
     with pytest.raises(ValueError, match=msg):
         imputer = SimpleImputer(strategy=strategy)
         imputer.fit_transform(X)
@@ -319,7 +317,8 @@ def test_imputation_const_mostf_error_invalid_types(strategy, dtype):
         imputer.fit(X).transform(X)
 
 
-def test_imputation_most_frequent():
+@pytest.mark.parametrize("csc_container", CSC_CONTAINERS)
+def test_imputation_most_frequent(csc_container):
     # Test imputation using the most-frequent strategy.
     X = np.array(
         [
@@ -343,7 +342,7 @@ def test_imputation_most_frequent():
     # frequent as promised in the doc but the lowest most frequent. When this
     # test will fail after an update of scipy, SimpleImputer will need to be
     # updated to be consistent with the new (correct) behaviour
-    _check_statistics(X, X_true, "most_frequent", [np.nan, 2, 3, 3], -1)
+    _check_statistics(X, X_true, "most_frequent", [np.nan, 2, 3, 3], -1, csc_container)
 
 
 @pytest.mark.parametrize("marker", [None, np.nan, "NAN", "", 0])
@@ -420,7 +419,7 @@ def test_imputation_constant_integer():
     assert_array_equal(X_trans, X_true)
 
 
-@pytest.mark.parametrize("array_constructor", [sparse.csr_matrix, np.asarray])
+@pytest.mark.parametrize("array_constructor", CSR_CONTAINERS + [np.asarray])
 def test_imputation_constant_float(array_constructor):
     # Test imputation using the constant strategy on floats
     X = np.array(
@@ -917,20 +916,6 @@ def test_iterative_imputer_additive_matrix():
     assert_allclose(X_test_filled, X_test_est, rtol=1e-3, atol=0.01)
 
 
-@pytest.mark.parametrize(
-    "max_iter, tol, error_type, warning",
-    [
-        (-1, 1e-3, ValueError, "should be a positive integer"),
-        (1, -1e-3, ValueError, "should be a non-negative float"),
-    ],
-)
-def test_iterative_imputer_error_param(max_iter, tol, error_type, warning):
-    X = np.zeros((100, 2))
-    imputer = IterativeImputer(max_iter=max_iter, tol=tol)
-    with pytest.raises(error_type, match=warning):
-        imputer.fit_transform(X)
-
-
 def test_iterative_imputer_early_stopping():
     rng = np.random.RandomState(0)
     n = 50
@@ -1113,18 +1098,6 @@ def test_iterative_imputer_dont_set_random_state(rs_imputer, rs_estimator):
             {"features": "missing-only", "sparse": "auto"},
             "have missing values in transform but have no missing values in fit",
         ),
-        (
-            np.array([[-1, 1], [1, 2]]),
-            np.array([[-1, 1], [1, 2]]),
-            {"features": "random", "sparse": "auto"},
-            "'features' has to be either 'missing-only' or 'all'",
-        ),
-        (
-            np.array([[-1, 1], [1, 2]]),
-            np.array([[-1, 1], [1, 2]]),
-            {"features": "all", "sparse": "random"},
-            "'sparse' has to be a boolean or 'auto'",
-        ),
         (
             np.array([["a", "b"], ["c", "a"]], dtype=str),
             np.array([["a", "b"], ["c", "a"]], dtype=str),
@@ -1140,23 +1113,27 @@ def test_missing_indicator_error(X_fit, X_trans, params, msg_err):
         indicator.fit(X_fit).transform(X_trans)
 
 
+def _generate_missing_indicator_cases():
+    missing_values_dtypes = [(0, np.int32), (np.nan, np.float64), (-1, np.int32)]
+    arr_types = (
+        [np.array]
+        + CSC_CONTAINERS
+        + CSR_CONTAINERS
+        + COO_CONTAINERS
+        + LIL_CONTAINERS
+        + BSR_CONTAINERS
+    )
+    return [
+        (arr_type, missing_values, dtype)
+        for arr_type, (missing_values, dtype) in product(
+            arr_types, missing_values_dtypes
+        )
+        if not (missing_values == 0 and arr_type is not np.array)
+    ]
+
+
 @pytest.mark.parametrize(
-    "missing_values, dtype, arr_type",
-    [
-        (np.nan, np.float64, np.array),
-        (0, np.int32, np.array),
-        (-1, np.int32, np.array),
-        (np.nan, np.float64, sparse.csc_matrix),
-        (-1, np.int32, sparse.csc_matrix),
-        (np.nan, np.float64, sparse.csr_matrix),
-        (-1, np.int32, sparse.csr_matrix),
-        (np.nan, np.float64, sparse.coo_matrix),
-        (-1, np.int32, sparse.coo_matrix),
-        (np.nan, np.float64, sparse.lil_matrix),
-        (-1, np.int32, sparse.lil_matrix),
-        (np.nan, np.float64, sparse.bsr_matrix),
-        (-1, np.int32, sparse.bsr_matrix),
-    ],
+    "arr_type, missing_values, dtype", _generate_missing_indicator_cases()
 )
 @pytest.mark.parametrize(
     "param_features, n_features, features_indices",
@@ -1208,13 +1185,7 @@ def test_missing_indicator_new(
 
 @pytest.mark.parametrize(
     "arr_type",
-    [
-        sparse.csc_matrix,
-        sparse.csr_matrix,
-        sparse.coo_matrix,
-        sparse.lil_matrix,
-        sparse.bsr_matrix,
-    ],
+    CSC_CONTAINERS + CSR_CONTAINERS + COO_CONTAINERS + LIL_CONTAINERS + BSR_CONTAINERS,
 )
 def test_missing_indicator_raise_on_sparse_with_missing_0(arr_type):
     # test for sparse input and missing_value == 0
@@ -1239,15 +1210,18 @@ def test_missing_indicator_raise_on_sparse_with_missing_0(arr_type):
 
 @pytest.mark.parametrize("param_sparse", [True, False, "auto"])
 @pytest.mark.parametrize(
-    "missing_values, arr_type",
-    [
-        (np.nan, np.array),
-        (0, np.array),
-        (np.nan, sparse.csc_matrix),
-        (np.nan, sparse.csr_matrix),
-        (np.nan, sparse.coo_matrix),
-        (np.nan, sparse.lil_matrix),
-    ],
+    "arr_type, missing_values",
+    [(np.array, 0)]
+    + list(
+        product(
+            CSC_CONTAINERS
+            + CSR_CONTAINERS
+            + COO_CONTAINERS
+            + LIL_CONTAINERS
+            + BSR_CONTAINERS,
+            [np.nan],
+        )
+    ),
 )
 def test_missing_indicator_sparse_param(arr_type, missing_values, param_sparse):
     # check the format of the output with different sparse parameter
@@ -1353,10 +1327,11 @@ def test_missing_indicator_no_missing():
     assert Xt.shape[1] == 0
 
 
-def test_missing_indicator_sparse_no_explicit_zeros():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_missing_indicator_sparse_no_explicit_zeros(csr_container):
     # Check that non missing values don't become explicit zeros in the mask
     # generated by missing indicator when X is sparse. (#13491)
-    X = sparse.csr_matrix([[0, 1, 2], [1, 2, 0], [2, 0, 1]])
+    X = csr_container([[0, 1, 2], [1, 2, 0], [2, 0, 1]])
 
     mi = MissingIndicator(features="all", missing_values=1)
     Xt = mi.fit_transform(X)
@@ -1375,13 +1350,7 @@ def test_imputer_without_indicator(imputer_constructor):
 
 @pytest.mark.parametrize(
     "arr_type",
-    [
-        sparse.csc_matrix,
-        sparse.csr_matrix,
-        sparse.coo_matrix,
-        sparse.lil_matrix,
-        sparse.bsr_matrix,
-    ],
+    CSC_CONTAINERS + CSR_CONTAINERS + COO_CONTAINERS + LIL_CONTAINERS + BSR_CONTAINERS,
 )
 def test_simple_imputation_add_indicator_sparse_matrix(arr_type):
     X_sparse = arr_type([[np.nan, 1, 5], [2, np.nan, 1], [6, 3, np.nan], [1, 2, 9]])
@@ -1541,6 +1510,55 @@ def test_most_frequent(expected, array, dtype, extra_value, n_repeat):
     )
 
 
+@pytest.mark.parametrize(
+    "initial_strategy", ["mean", "median", "most_frequent", "constant"]
+)
+def test_iterative_imputer_keep_empty_features(initial_strategy):
+    """Check the behaviour of the iterative imputer with different initial strategy
+    and keeping empty features (i.e. features containing only missing values).
+    """
+    X = np.array([[1, np.nan, 2], [3, np.nan, np.nan]])
+
+    imputer = IterativeImputer(
+        initial_strategy=initial_strategy, keep_empty_features=True
+    )
+    X_imputed = imputer.fit_transform(X)
+    assert_allclose(X_imputed[:, 1], 0)
+    X_imputed = imputer.transform(X)
+    assert_allclose(X_imputed[:, 1], 0)
+
+
+def test_iterative_imputer_constant_fill_value():
+    """Check that we propagate properly the parameter `fill_value`."""
+    X = np.array([[-1, 2, 3, -1], [4, -1, 5, -1], [6, 7, -1, -1], [8, 9, 0, -1]])
+
+    fill_value = 100
+    imputer = IterativeImputer(
+        missing_values=-1,
+        initial_strategy="constant",
+        fill_value=fill_value,
+        max_iter=0,
+    )
+    imputer.fit_transform(X)
+    assert_array_equal(imputer.initial_imputer_.statistics_, fill_value)
+
+
+@pytest.mark.parametrize("keep_empty_features", [True, False])
+def test_knn_imputer_keep_empty_features(keep_empty_features):
+    """Check the behaviour of `keep_empty_features` for `KNNImputer`."""
+    X = np.array([[1, np.nan, 2], [3, np.nan, np.nan]])
+
+    imputer = KNNImputer(keep_empty_features=keep_empty_features)
+
+    for method in ["fit_transform", "transform"]:
+        X_imputed = getattr(imputer, method)(X)
+        if keep_empty_features:
+            assert X_imputed.shape == X.shape
+            assert_array_equal(X_imputed[:, 1], 0)
+        else:
+            assert X_imputed.shape == (X.shape[0], X.shape[1] - 1)
+
+
 def test_simple_impute_pd_na():
     pd = pytest.importorskip("pandas")
 
@@ -1644,3 +1662,51 @@ def test_imputer_transform_preserves_numeric_dtype(dtype_test):
     X_test = np.asarray([[np.nan, np.nan, np.nan]], dtype=dtype_test)
     X_trans = imp.transform(X_test)
     assert X_trans.dtype == dtype_test
+
+
+@pytest.mark.parametrize("array_type", ["array", "sparse"])
+@pytest.mark.parametrize("keep_empty_features", [True, False])
+def test_simple_imputer_constant_keep_empty_features(array_type, keep_empty_features):
+    """Check the behaviour of `keep_empty_features` with `strategy='constant'.
+    For backward compatibility, a column full of missing values will always be
+    fill and never dropped.
+    """
+    X = np.array([[np.nan, 2], [np.nan, 3], [np.nan, 6]])
+    X = _convert_container(X, array_type)
+    fill_value = 10
+    imputer = SimpleImputer(
+        strategy="constant",
+        fill_value=fill_value,
+        keep_empty_features=keep_empty_features,
+    )
+
+    for method in ["fit_transform", "transform"]:
+        X_imputed = getattr(imputer, method)(X)
+        assert X_imputed.shape == X.shape
+        constant_feature = (
+            X_imputed[:, 0].toarray() if array_type == "sparse" else X_imputed[:, 0]
+        )
+        assert_array_equal(constant_feature, fill_value)
+
+
+@pytest.mark.parametrize("array_type", ["array", "sparse"])
+@pytest.mark.parametrize("strategy", ["mean", "median", "most_frequent"])
+@pytest.mark.parametrize("keep_empty_features", [True, False])
+def test_simple_imputer_keep_empty_features(strategy, array_type, keep_empty_features):
+    """Check the behaviour of `keep_empty_features` with all strategies but
+    'constant'.
+    """
+    X = np.array([[np.nan, 2], [np.nan, 3], [np.nan, 6]])
+    X = _convert_container(X, array_type)
+    imputer = SimpleImputer(strategy=strategy, keep_empty_features=keep_empty_features)
+
+    for method in ["fit_transform", "transform"]:
+        X_imputed = getattr(imputer, method)(X)
+        if keep_empty_features:
+            assert X_imputed.shape == X.shape
+            constant_feature = (
+                X_imputed[:, 0].toarray() if array_type == "sparse" else X_imputed[:, 0]
+            )
+            assert_array_equal(constant_feature, 0)
+        else:
+            assert X_imputed.shape == (X.shape[0], X.shape[1] - 1)
