@@ -2,35 +2,34 @@
 #          Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
 # License: BSD 3 clause
 
-import numpy as np
 import warnings
+from numbers import Real
 
-from ._base import NeighborsBase
-from ._base import KNeighborsMixin
-from ._base import UnsupervisedMixin
-from ..base import OutlierMixin
+import numpy as np
 
-from ..utils.validation import check_is_fitted
-from ..utils.validation import _deprecate_positional_args
+from ..base import OutlierMixin, _fit_context
 from ..utils import check_array
+from ..utils._param_validation import Interval, StrOptions
+from ..utils.metaestimators import available_if
+from ..utils.validation import check_is_fitted
+from ._base import KNeighborsMixin, NeighborsBase
 
 __all__ = ["LocalOutlierFactor"]
 
 
-class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
-                         OutlierMixin, NeighborsBase):
-    """Unsupervised Outlier Detection using Local Outlier Factor (LOF)
+class LocalOutlierFactor(KNeighborsMixin, OutlierMixin, NeighborsBase):
+    """Unsupervised Outlier Detection using the Local Outlier Factor (LOF).
 
-    The anomaly score of each sample is called Local Outlier Factor.
-    It measures the local deviation of density of a given sample with
-    respect to its neighbors.
+    The anomaly score of each sample is called the Local Outlier Factor.
+    It measures the local deviation of the density of a given sample with respect
+    to its neighbors.
     It is local in that the anomaly score depends on how isolated the object
     is with respect to the surrounding neighborhood.
     More precisely, locality is given by k-nearest neighbors, whose distance
     is used to estimate the local density.
-    By comparing the local density of a sample to the local densities of
-    its neighbors, one can identify samples that have a substantially lower
-    density than their neighbors. These are considered outliers.
+    By comparing the local density of a sample to the local densities of its
+    neighbors, one can identify samples that have a substantially lower density
+    than their neighbors. These are considered outliers.
 
     .. versionadded:: 0.19
 
@@ -54,43 +53,32 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
         this parameter, using brute force.
 
     leaf_size : int, default=30
-        Leaf size passed to :class:`BallTree` or :class:`KDTree`. This can
+        Leaf is size passed to :class:`BallTree` or :class:`KDTree`. This can
         affect the speed of the construction and query, as well as the memory
         required to store the tree. The optimal value depends on the
         nature of the problem.
 
     metric : str or callable, default='minkowski'
-        metric used for the distance computation. Any metric from scikit-learn
-        or scipy.spatial.distance can be used.
+        Metric to use for distance computation. Default is "minkowski", which
+        results in the standard Euclidean distance when p = 2. See the
+        documentation of `scipy.spatial.distance
+        <https://docs.scipy.org/doc/scipy/reference/spatial.distance.html>`_ and
+        the metrics listed in
+        :class:`~sklearn.metrics.pairwise.distance_metrics` for valid metric
+        values.
 
         If metric is "precomputed", X is assumed to be a distance matrix and
-        must be square. X may be a sparse matrix, in which case only "nonzero"
-        elements may be considered neighbors.
+        must be square during fit. X may be a :term:`sparse graph`, in which
+        case only "nonzero" elements may be considered neighbors.
 
-        If metric is a callable function, it is called on each
-        pair of instances (rows) and the resulting value recorded. The callable
-        should take two arrays as input and return one value indicating the
-        distance between them. This works for Scipy's metrics, but is less
+        If metric is a callable function, it takes two arrays representing 1D
+        vectors as inputs and must return one value indicating the distance
+        between those vectors. This works for Scipy's metrics, but is less
         efficient than passing the metric name as a string.
 
-        Valid values for metric are:
-
-        - from scikit-learn: ['cityblock', 'cosine', 'euclidean', 'l1', 'l2',
-          'manhattan']
-
-        - from scipy.spatial.distance: ['braycurtis', 'canberra', 'chebyshev',
-          'correlation', 'dice', 'hamming', 'jaccard', 'kulsinski',
-          'mahalanobis', 'minkowski', 'rogerstanimoto', 'russellrao',
-          'seuclidean', 'sokalmichener', 'sokalsneath', 'sqeuclidean',
-          'yule']
-
-        See the documentation for scipy.spatial.distance for details on these
-        metrics:
-        https://docs.scipy.org/doc/scipy/reference/spatial.distance.html
-
-    p : int, default=2
+    p : float, default=2
         Parameter for the Minkowski metric from
-        :func:`sklearn.metrics.pairwise.pairwise_distances`. When p = 1, this
+        :func:`sklearn.metrics.pairwise_distances`. When p = 1, this
         is equivalent to using manhattan_distance (l1), and euclidean_distance
         (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
 
@@ -104,7 +92,7 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
 
         - if 'auto', the threshold is determined as in the
           original paper,
-        - if a float, the contamination should be in the range [0, 0.5].
+        - if a float, the contamination should be in the range (0, 0.5].
 
         .. versionchanged:: 0.22
            The default value of ``contamination`` changed from 0.1
@@ -114,8 +102,9 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
         By default, LocalOutlierFactor is only meant to be used for outlier
         detection (novelty=False). Set novelty to True if you want to use
         LocalOutlierFactor for novelty detection. In this case be aware that
-        that you should only use predict, decision_function and score_samples
-        on new unseen data and not on the training set.
+        you should only use predict, decision_function and score_samples
+        on new unseen data and not on the training set; and note that the
+        results obtained this way may differ from the standard LOF results.
 
         .. versionadded:: 0.20
 
@@ -152,6 +141,36 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
 
         .. versionadded:: 0.20
 
+    effective_metric_ : str
+        The effective metric used for the distance computation.
+
+    effective_metric_params_ : dict
+        The effective additional keyword arguments for the metric function.
+
+    n_features_in_ : int
+        Number of features seen during :term:`fit`.
+
+        .. versionadded:: 0.24
+
+    feature_names_in_ : ndarray of shape (`n_features_in_`,)
+        Names of features seen during :term:`fit`. Defined only when `X`
+        has feature names that are all strings.
+
+        .. versionadded:: 1.0
+
+    n_samples_fit_ : int
+        It is the number of samples in the fitted data.
+
+    See Also
+    --------
+    sklearn.svm.OneClassSVM: Unsupervised Outlier Detection using
+        Support Vector Machine.
+
+    References
+    ----------
+    .. [1] Breunig, M. M., Kriegel, H. P., Ng, R. T., & Sander, J. (2000, May).
+           LOF: identifying density-based local outliers. In ACM sigmod record.
+
     Examples
     --------
     >>> import numpy as np
@@ -162,37 +181,65 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
     array([ 1,  1, -1,  1])
     >>> clf.negative_outlier_factor_
     array([ -0.9821...,  -1.0370..., -73.3697...,  -0.9821...])
-
-    References
-    ----------
-    .. [1] Breunig, M. M., Kriegel, H. P., Ng, R. T., & Sander, J. (2000, May).
-           LOF: identifying density-based local outliers. In ACM sigmod record.
     """
-    @_deprecate_positional_args
-    def __init__(self, n_neighbors=20, *, algorithm='auto', leaf_size=30,
-                 metric='minkowski', p=2, metric_params=None,
-                 contamination="auto", novelty=False, n_jobs=None):
+
+    _parameter_constraints: dict = {
+        **NeighborsBase._parameter_constraints,
+        "contamination": [
+            StrOptions({"auto"}),
+            Interval(Real, 0, 0.5, closed="right"),
+        ],
+        "novelty": ["boolean"],
+    }
+    _parameter_constraints.pop("radius")
+
+    def __init__(
+        self,
+        n_neighbors=20,
+        *,
+        algorithm="auto",
+        leaf_size=30,
+        metric="minkowski",
+        p=2,
+        metric_params=None,
+        contamination="auto",
+        novelty=False,
+        n_jobs=None,
+    ):
         super().__init__(
             n_neighbors=n_neighbors,
             algorithm=algorithm,
-            leaf_size=leaf_size, metric=metric, p=p,
-            metric_params=metric_params, n_jobs=n_jobs)
+            leaf_size=leaf_size,
+            metric=metric,
+            p=p,
+            metric_params=metric_params,
+            n_jobs=n_jobs,
+        )
         self.contamination = contamination
         self.novelty = novelty
 
-    @property
-    def fit_predict(self):
-        """Fits the model to the training set X and returns the labels.
+    def _check_novelty_fit_predict(self):
+        if self.novelty:
+            msg = (
+                "fit_predict is not available when novelty=True. Use "
+                "novelty=False if you want to predict on the training set."
+            )
+            raise AttributeError(msg)
+        return True
 
-        **Only available for novelty detection (when novelty is set to True).**
+    @available_if(_check_novelty_fit_predict)
+    def fit_predict(self, X, y=None):
+        """Fit the model to the training set X and return the labels.
+
+        **Not available for novelty detection (when novelty is set to True).**
         Label is 1 for an inlier and -1 for an outlier according to the LOF
         score and the contamination parameter.
 
         Parameters
         ----------
-        X : array-like of shape (n_samples, n_features), default=None
+        X : {array-like, sparse matrix} of shape (n_samples, n_features), default=None
             The query sample or samples to compute the Local Outlier Factor
-            w.r.t. to the training samples.
+            w.r.t. the training samples.
 
         y : Ignored
             Not used, present for API consistency by convention.
@@ -206,77 +253,59 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
         # As fit_predict would be different from fit.predict, fit_predict is
         # only available for outlier detection (novelty=False)
 
-        if self.novelty:
-            msg = ('fit_predict is not available when novelty=True. Use '
-                   'novelty=False if you want to predict on the training set.')
-            raise AttributeError(msg)
-
-        return self._fit_predict
-
-    def _fit_predict(self, X, y=None):
-        """Fits the model to the training set X and returns the labels.
-
-        Label is 1 for an inlier and -1 for an outlier according to the LOF
-        score and the contamination parameter.
-
-        Parameters
-        ----------
-        X : array-like of shape (n_samples, n_features), default=None
-            The query sample or samples to compute the Local Outlier Factor
-            w.r.t. to the training samples.
-
-        Returns
-        -------
-        is_inlier : ndarray of shape (n_samples,)
-            Returns -1 for anomalies/outliers and 1 for inliers.
-        """
-
-        # As fit_predict would be different from fit.predict, fit_predict is
-        # only available for outlier detection (novelty=False)
-
         return self.fit(X)._predict()
 
+    @_fit_context(
+        # LocalOutlierFactor.metric is not validated yet
+        prefer_skip_nested_validation=False
+    )
     def fit(self, X, y=None):
-        """Fit the model using X as training data.
+        """Fit the local outlier factor detector from the training dataset.
 
         Parameters
         ----------
-        X : BallTree, KDTree or {array-like, sparse matrix} of shape \
-                (n_samples, n_features) or (n_samples, n_samples)
-            Training data. If array or matrix, the shape is (n_samples,
-            n_features), or (n_samples, n_samples) if metric='precomputed'.
+        X : {array-like, sparse matrix} of shape (n_samples, n_features) or \
+                (n_samples, n_samples) if metric='precomputed'
+            Training data.
 
         y : Ignored
             Not used, present for API consistency by convention.
 
         Returns
         -------
-        self : object
+        self : LocalOutlierFactor
+            The fitted local outlier factor detector.
         """
-        if self.contamination != 'auto':
-            if not(0. < self.contamination <= .5):
-                raise ValueError("contamination must be in (0, 0.5], "
-                                 "got: %f" % self.contamination)
-
-        super().fit(X)
+        self._fit(X)
 
         n_samples = self.n_samples_fit_
         if self.n_neighbors > n_samples:
-            warnings.warn("n_neighbors (%s) is greater than the "
-                          "total number of samples (%s). n_neighbors "
-                          "will be set to (n_samples - 1) for estimation."
-                          % (self.n_neighbors, n_samples))
+            warnings.warn(
+                "n_neighbors (%s) is greater than the "
+                "total number of samples (%s). n_neighbors "
+                "will be set to (n_samples - 1) for estimation."
+                % (self.n_neighbors, n_samples)
+            )
         self.n_neighbors_ = max(1, min(self.n_neighbors, n_samples - 1))
 
         self._distances_fit_X_, _neighbors_indices_fit_X_ = self.kneighbors(
-            n_neighbors=self.n_neighbors_)
+            n_neighbors=self.n_neighbors_
+        )
+
+        if self._fit_X.dtype == np.float32:
+            self._distances_fit_X_ = self._distances_fit_X_.astype(
+                self._fit_X.dtype,
+                copy=False,
+            )
 
         self._lrd = self._local_reachability_density(
-            self._distances_fit_X_, _neighbors_indices_fit_X_)
+            self._distances_fit_X_, _neighbors_indices_fit_X_
+        )
 
         # Compute lof score over training samples to define offset_:
-        lrd_ratios_array = (self._lrd[_neighbors_indices_fit_X_] /
-                            self._lrd[:, np.newaxis])
+        lrd_ratios_array = (
+            self._lrd[_neighbors_indices_fit_X_] / self._lrd[:, np.newaxis]
+        )
 
         self.negative_outlier_factor_ = -np.mean(lrd_ratios_array, axis=1)
 
@@ -284,38 +313,45 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
             # inliers score around -1 (the higher, the less abnormal).
             self.offset_ = -1.5
         else:
-            self.offset_ = np.percentile(self.negative_outlier_factor_,
-                                         100. * self.contamination)
+            self.offset_ = np.percentile(
+                self.negative_outlier_factor_, 100.0 * self.contamination
+            )
 
         return self
 
-    @property
-    def predict(self):
+    def _check_novelty_predict(self):
+        if not self.novelty:
+            msg = (
+                "predict is not available when novelty=False, use "
+                "fit_predict if you want to predict on training data. Use "
+                "novelty=True if you want to use LOF for novelty detection "
+                "and predict on new unseen data."
+            )
+            raise AttributeError(msg)
+        return True
+
+    @available_if(_check_novelty_predict)
+    def predict(self, X=None):
         """Predict the labels (1 inlier, -1 outlier) of X according to LOF.
 
         **Only available for novelty detection (when novelty is set to True).**
         This method allows to generalize prediction to *new observations* (not
-        in the training set).
+        in the training set). Note that the result of ``clf.fit(X)`` then
+        ``clf.predict(X)`` with ``novelty=True`` may differ from the result
+        obtained by ``clf.fit_predict(X)`` with ``novelty=False``.
 
         Parameters
         ----------
-        X : array-like of shape (n_samples, n_features)
+        X : {array-like, sparse matrix} of shape (n_samples, n_features)
             The query sample or samples to compute the Local Outlier Factor
-            w.r.t. to the training samples.
+            w.r.t. the training samples.
 
         Returns
         -------
         is_inlier : ndarray of shape (n_samples,)
             Returns -1 for anomalies/outliers and +1 for inliers.
         """
-        if not self.novelty:
-            msg = ('predict is not available when novelty=False, use '
-                   'fit_predict if you want to predict on training data. Use '
-                   'novelty=True if you want to use LOF for novelty detection '
-                   'and predict on new unseen data.')
-            raise AttributeError(msg)
-
-        return self._predict
+        return self._predict(X)
 
     def _predict(self, X=None):
         """Predict the labels (1 inlier, -1 outlier) of X according to LOF.
@@ -324,9 +360,9 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
 
         Parameters
         ----------
-        X : array-like of shape (n_samples, n_features), default=None
+        X : {array-like, sparse matrix} of shape (n_samples, n_features), default=None
             The query sample or samples to compute the Local Outlier Factor
-            w.r.t. to the training samples. If None, makes prediction on the
+            w.r.t. the training samples. If None, makes prediction on the
             training data without considering them as their own neighbors.
 
         Returns
@@ -337,7 +373,7 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
         check_is_fitted(self)
 
         if X is not None:
-            X = check_array(X, accept_sparse='csr')
+            X = check_array(X, accept_sparse="csr")
             is_inlier = np.ones(X.shape[0], dtype=int)
             is_inlier[self.decision_function(X) < 0] = -1
         else:
@@ -346,8 +382,21 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
 
         return is_inlier
 
-    @property
-    def decision_function(self):
+    def _check_novelty_decision_function(self):
+        if not self.novelty:
+            msg = (
+                "decision_function is not available when novelty=False. "
+                "Use novelty=True if you want to use LOF for novelty "
+                "detection and compute decision_function for new unseen "
+                "data. Note that the opposite LOF of the training samples "
+                "is always available by considering the "
+                "negative_outlier_factor_ attribute."
+            )
+            raise AttributeError(msg)
+        return True
+
+    @available_if(_check_novelty_decision_function)
+    def decision_function(self, X):
         """Shifted opposite of the Local Outlier Factor of X.
 
         Bigger is better, i.e. large values correspond to inliers.
@@ -361,7 +410,7 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
 
         Parameters
         ----------
-        X : array-like of shape (n_samples, n_features)
+        X : {array-like, sparse matrix} of shape (n_samples, n_features)
             The query sample or samples to compute the Local Outlier Factor
             w.r.t. the training samples.
 
@@ -372,47 +421,22 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
             samples. The lower, the more abnormal. Negative scores represent
             outliers, positive scores represent inliers.
         """
+        return self.score_samples(X) - self.offset_
+
+    def _check_novelty_score_samples(self):
         if not self.novelty:
-            msg = ('decision_function is not available when novelty=False. '
-                   'Use novelty=True if you want to use LOF for novelty '
-                   'detection and compute decision_function for new unseen '
-                   'data. Note that the opposite LOF of the training samples '
-                   'is always available by considering the '
-                   'negative_outlier_factor_ attribute.')
+            msg = (
+                "score_samples is not available when novelty=False. The "
+                "scores of the training samples are always available "
+                "through the negative_outlier_factor_ attribute. Use "
+                "novelty=True if you want to use LOF for novelty detection "
+                "and compute score_samples for new unseen data."
+            )
             raise AttributeError(msg)
+        return True
 
-        return self._decision_function
-
-    def _decision_function(self, X):
-        """Shifted opposite of the Local Outlier Factor of X.
-
-        Bigger is better, i.e. large values correspond to inliers.
-
-        **Only available for novelty detection (when novelty is set to True).**
-        The shift offset allows a zero threshold for being an outlier.
-        The argument X is supposed to contain *new data*: if X contains a
-        point from training, it considers the later in its own neighborhood.
-        Also, the samples in X are not considered in the neighborhood of any
-        point.
-
-        Parameters
-        ----------
-        X : array-like of shape (n_samples, n_features)
-            The query sample or samples to compute the Local Outlier Factor
-            w.r.t. the training samples.
-
-        Returns
-        -------
-        shifted_opposite_lof_scores : ndarray of shape (n_samples,)
-            The shifted opposite of the Local Outlier Factor of each input
-            samples. The lower, the more abnormal. Negative scores represent
-            outliers, positive scores represent inliers.
-        """
-
-        return self._score_samples(X) - self.offset_
-
-    @property
-    def score_samples(self):
+    @available_if(_check_novelty_score_samples)
+    def score_samples(self, X):
         """Opposite of the Local Outlier Factor of X.
 
         It is the opposite as bigger is better, i.e. large values correspond
@@ -422,49 +446,14 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
         The argument X is supposed to contain *new data*: if X contains a
         point from training, it considers the later in its own neighborhood.
         Also, the samples in X are not considered in the neighborhood of any
-        point.
-        The score_samples on training data is available by considering the
-        the ``negative_outlier_factor_`` attribute.
+        point. Because of this, the scores obtained via ``score_samples`` may
+        differ from the standard LOF scores.
+        The standard LOF scores for the training data is available via the
+        ``negative_outlier_factor_`` attribute.
 
         Parameters
         ----------
-        X : array-like of shape (n_samples, n_features)
-            The query sample or samples to compute the Local Outlier Factor
-            w.r.t. the training samples.
-
-        Returns
-        -------
-        opposite_lof_scores : ndarray of shape (n_samples,)
-            The opposite of the Local Outlier Factor of each input samples.
-            The lower, the more abnormal.
-        """
-        if not self.novelty:
-            msg = ('score_samples is not available when novelty=False. The '
-                   'scores of the training samples are always available '
-                   'through the negative_outlier_factor_ attribute. Use '
-                   'novelty=True if you want to use LOF for novelty detection '
-                   'and compute score_samples for new unseen data.')
-            raise AttributeError(msg)
-
-        return self._score_samples
-
-    def _score_samples(self, X):
-        """Opposite of the Local Outlier Factor of X.
-
-        It is the opposite as bigger is better, i.e. large values correspond
-        to inliers.
-
-        **Only available for novelty detection (when novelty is set to True).**
-        The argument X is supposed to contain *new data*: if X contains a
-        point from training, it considers the later in its own neighborhood.
-        Also, the samples in X are not considered in the neighborhood of any
-        point.
-        The score_samples on training data is available by considering the
-        the ``negative_outlier_factor_`` attribute.
-
-        Parameters
-        ----------
-        X : array-like of shape (n_samples, n_features)
+        X : {array-like, sparse matrix} of shape (n_samples, n_features)
             The query sample or samples to compute the Local Outlier Factor
             w.r.t. the training samples.
 
@@ -475,15 +464,21 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
             The lower, the more abnormal.
         """
         check_is_fitted(self)
-        X = check_array(X, accept_sparse='csr')
+        X = check_array(X, accept_sparse="csr")
 
-        distances_X, neighbors_indices_X = (
-            self.kneighbors(X, n_neighbors=self.n_neighbors_))
-        X_lrd = self._local_reachability_density(distances_X,
-                                                 neighbors_indices_X)
+        distances_X, neighbors_indices_X = self.kneighbors(
+            X, n_neighbors=self.n_neighbors_
+        )
 
-        lrd_ratios_array = (self._lrd[neighbors_indices_X] /
-                            X_lrd[:, np.newaxis])
+        if X.dtype == np.float32:
+            distances_X = distances_X.astype(X.dtype, copy=False)
+
+        X_lrd = self._local_reachability_density(
+            distances_X,
+            neighbors_indices_X,
+        )
+
+        lrd_ratios_array = self._lrd[neighbors_indices_X] / X_lrd[:, np.newaxis]
 
         # as bigger is better:
         return -np.mean(lrd_ratios_array, axis=1)
@@ -509,9 +504,13 @@ class LocalOutlierFactor(KNeighborsMixin, UnsupervisedMixin,
         local_reachability_density : ndarray of shape (n_queries,)
             The local reachability density of each sample.
         """
-        dist_k = self._distances_fit_X_[neighbors_indices,
-                                        self.n_neighbors_ - 1]
+        dist_k = self._distances_fit_X_[neighbors_indices, self.n_neighbors_ - 1]
         reach_dist_array = np.maximum(distances_X, dist_k)
 
         # 1e-10 to avoid `nan' when nb of duplicates > n_neighbors_:
-        return 1. / (np.mean(reach_dist_array, axis=1) + 1e-10)
+        return 1.0 / (np.mean(reach_dist_array, axis=1) + 1e-10)
+
+    def _more_tags(self):
+        return {
+            "preserves_dtype": [np.float64, np.float32],
+        }
