@@ -4,10 +4,12 @@ import numpy as np
 
 from ..base import BaseEstimator, TransformerMixin, _fit_context
 from ..utils._param_validation import StrOptions
+from ..utils._set_output import _get_output_config
 from ..utils.metaestimators import available_if
 from ..utils.validation import (
     _allclose_dense_sparse,
     _check_feature_names_in,
+    _is_pandas_df,
     check_array,
 )
 
@@ -237,7 +239,41 @@ class FunctionTransformer(TransformerMixin, BaseEstimator):
             Transformed input.
         """
         X = self._check_input(X, reset=False)
-        return self._transform(X, func=self.func, kw_args=self.kw_args)
+        out = self._transform(X, func=self.func, kw_args=self.kw_args)
+
+        if hasattr(out, "columns") and self.feature_names_out is not None:
+            # check the consistency between the column names of the output and the
+            # one generated by `get_feature_names_out`
+            if list(out.columns) != list(self.get_feature_names_out()):
+                raise ValueError(
+                    "The output generated by `func` have different column names than "
+                    "the one generated by the method `get_feature_names_out`. "
+                    f"Got output with columns names: {list(out.columns)} and "
+                    "`get_feature_names_out` returned: "
+                    f"{list(self.get_feature_names_out())}. "
+                    "This can be fixed in different manners depending on your use case:"
+                    "\n(i) If `func` returns a container with column names, make sure "
+                    "they are consistent with the output of `get_feature_names_out`.\n"
+                    "(ii) If `func` is a NumPy `ufunc`, then forcing `validate=True` "
+                    "could be considered to internally convert the input container to "
+                    "a NumPy array before calling the `ufunc`.\n"
+                    "(iii) The column names can be overriden by setting "
+                    "`set_output(transform='pandas')` such that the column names are "
+                    "set to the names provided by `get_feature_names_out`."
+                )
+
+        output_config = _get_output_config("transform", self)["dense"]
+        if (
+            output_config == "pandas"
+            and self.feature_names_out is None
+            and not _is_pandas_df(out)
+        ):
+            warnings.warn(
+                "When `set_output` is configured to be 'pandas', `func` should return "
+                "a DataFrame to follow the `set_output` API  or `feature_names_out` "
+                "should be defined."
+            )
+        return out
 
     def inverse_transform(self, X):
         """Transform X using the inverse function.
@@ -331,20 +367,19 @@ class FunctionTransformer(TransformerMixin, BaseEstimator):
 
             - `"default"`: Default output format of a transformer
             - `"pandas"`: DataFrame output
+            - `"polars"`: Polars output
             - `None`: Transform configuration is unchanged
 
+            .. versionadded:: 1.4
+                `"polars"` option was added.
+
         Returns
         -------
         self : estimator instance
             Estimator instance.
         """
-        if hasattr(super(), "set_output"):
-            return super().set_output(transform=transform)
-
-        if transform == "pandas" and self.feature_names_out is None:
-            warnings.warn(
-                'With transform="pandas", `func` should return a DataFrame to follow'
-                " the set_output API."
-            )
+        if not hasattr(self, "_sklearn_output_config"):
+            self._sklearn_output_config = {}
 
+        self._sklearn_output_config["transform"] = transform
         return self
