@@ -9,6 +9,7 @@ from scipy import sparse
 
 from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
 from sklearn.tree import DecisionTreeClassifier
+from sklearn.utils import _IS_WASM
 from sklearn.utils._testing import (
     TempMemmap,
     _convert_container,
@@ -26,6 +27,12 @@ from sklearn.utils._testing import (
     set_random_state,
 )
 from sklearn.utils.deprecation import deprecated
+from sklearn.utils.fixes import (
+    CSC_CONTAINERS,
+    CSR_CONTAINERS,
+    parse_version,
+    sp_version,
+)
 from sklearn.utils.metaestimators import available_if
 
 
@@ -38,10 +45,11 @@ def test_set_random_state():
     assert tree.random_state == 3
 
 
-def test_assert_allclose_dense_sparse():
+@pytest.mark.parametrize("csr_container", CSC_CONTAINERS)
+def test_assert_allclose_dense_sparse(csr_container):
     x = np.arange(9).reshape(3, 3)
     msg = "Not equal to tolerance "
-    y = sparse.csc_matrix(x)
+    y = csr_container(x)
     for X in [x, y]:
         # basic compare
         with pytest.raises(AssertionError, match=msg):
@@ -52,7 +60,7 @@ def test_assert_allclose_dense_sparse():
         assert_allclose_dense_sparse(x, y)
 
     A = sparse.diags(np.ones(5), offsets=0).tocsr()
-    B = sparse.csr_matrix(np.ones((1, 5)))
+    B = csr_container(np.ones((1, 5)))
     with pytest.raises(AssertionError, match="Arrays are not equal"):
         assert_allclose_dense_sparse(B, A)
 
@@ -600,42 +608,35 @@ def test_tempmemmap(monkeypatch):
     assert registration_counter.nb_calls == 2
 
 
-@pytest.mark.parametrize("aligned", [False, True])
-def test_create_memmap_backed_data(monkeypatch, aligned):
+@pytest.mark.xfail(_IS_WASM, reason="memmap not fully supported")
+def test_create_memmap_backed_data(monkeypatch):
     registration_counter = RegistrationCounter()
     monkeypatch.setattr(atexit, "register", registration_counter)
 
     input_array = np.ones(3)
-    data = create_memmap_backed_data(input_array, aligned=aligned)
+    data = create_memmap_backed_data(input_array)
     check_memmap(input_array, data)
     assert registration_counter.nb_calls == 1
 
-    data, folder = create_memmap_backed_data(
-        input_array, return_folder=True, aligned=aligned
-    )
+    data, folder = create_memmap_backed_data(input_array, return_folder=True)
     check_memmap(input_array, data)
     assert folder == os.path.dirname(data.filename)
     assert registration_counter.nb_calls == 2
 
     mmap_mode = "r+"
-    data = create_memmap_backed_data(input_array, mmap_mode=mmap_mode, aligned=aligned)
+    data = create_memmap_backed_data(input_array, mmap_mode=mmap_mode)
     check_memmap(input_array, data, mmap_mode)
     assert registration_counter.nb_calls == 3
 
     input_list = [input_array, input_array + 1, input_array + 2]
-    mmap_data_list = create_memmap_backed_data(input_list, aligned=aligned)
+    mmap_data_list = create_memmap_backed_data(input_list)
     for input_array, data in zip(input_list, mmap_data_list):
         check_memmap(input_array, data)
     assert registration_counter.nb_calls == 4
 
-    with pytest.raises(
-        ValueError,
-        match=(
-            "When creating aligned memmap-backed arrays, input must be a single array"
-            " or a sequence of arrays"
-        ),
-    ):
-        create_memmap_backed_data([input_array, "not-an-array"], aligned=True)
+    output_data, other = create_memmap_backed_data([input_array, "not-an-array"])
+    check_memmap(input_array, output_data)
+    assert other == "not-an-array"
 
 
 @pytest.mark.parametrize(
@@ -645,8 +646,10 @@ def test_create_memmap_backed_data(monkeypatch, aligned):
         ("tuple", tuple),
         ("array", np.ndarray),
         ("sparse", sparse.csr_matrix),
-        ("sparse_csr", sparse.csr_matrix),
-        ("sparse_csc", sparse.csc_matrix),
+        # using `zip` will only keep the available sparse containers
+        # depending of the installed SciPy version
+        *zip(["sparse_csr", "sparse_csr_array"], CSR_CONTAINERS),
+        *zip(["sparse_csc", "sparse_csc_array"], CSC_CONTAINERS),
         ("dataframe", lambda: pytest.importorskip("pandas").DataFrame),
         ("series", lambda: pytest.importorskip("pandas").Series),
         ("index", lambda: pytest.importorskip("pandas").Index),
@@ -675,6 +678,7 @@ def test_convert_container(
         # instead of the whole file
         container_type = container_type()
     container = [0, 1]
+
     container_converted = _convert_container(
         container,
         constructor_name,
@@ -692,6 +696,44 @@ def test_convert_container(
         assert container_converted.dtypes[0] == dtype
 
 
+def test_convert_container_categories_pandas():
+    pytest.importorskip("pandas")
+    df = _convert_container(
+        [["x"]], "dataframe", ["A"], categorical_feature_names=["A"]
+    )
+    assert df.dtypes.iloc[0] == "category"
+
+
+def test_convert_container_categories_polars():
+    pl = pytest.importorskip("polars")
+    df = _convert_container([["x"]], "polars", ["A"], categorical_feature_names=["A"])
+    assert df.schema["A"] == pl.Categorical()
+
+
+def test_convert_container_categories_pyarrow():
+    pa = pytest.importorskip("pyarrow")
+    df = _convert_container([["x"]], "pyarrow", ["A"], categorical_feature_names=["A"])
+    assert type(df.schema[0].type) is pa.DictionaryType
+
+
+@pytest.mark.skipif(
+    sp_version >= parse_version("1.8"),
+    reason="sparse arrays are available as of scipy 1.8.0",
+)
+@pytest.mark.parametrize("constructor_name", ["sparse_csr_array", "sparse_csc_array"])
+@pytest.mark.parametrize("dtype", [np.int32, np.int64, np.float32, np.float64])
+def test_convert_container_raise_when_sparray_not_available(constructor_name, dtype):
+    """Check that if we convert to sparse array but sparse array are not supported
+    (scipy<1.8.0), we should raise an explicit error."""
+    container = [0, 1]
+
+    with pytest.raises(
+        ValueError,
+        match=f"only available with scipy>=1.8.0, got {sp_version}",
+    ):
+        _convert_container(container, constructor_name, dtype=dtype)
+
+
 def test_raises():
     # Tests for the raises context manager
 
